-------------------------------------------------------------------------------
--! @file       counter_n_en_ld.vhd
--! @brief      n-bit counter with enable and load
--! @project    CAESAR Candidate Evaluation
--! @author     Michael X. Lyons
--! @copyright  Copyright (c) 2016 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity counter_n_en_ld is
    generic (
        n : integer
    );
    port (
        clk   : in  std_logic;
        en    : in  std_logic;
        ld    : in  std_logic;
        d     : in  unsigned((n - 1) downto 0);
        count : out unsigned((n - 1) downto 0)
    );
end entity counter_n_en_ld;

architecture behavioral of counter_n_en_ld is
    signal c : unsigned((n - 1) downto 0);
begin
    -- register
    process (clk)
    begin
        if rising_edge(clk) then
            if en = '1' then
                if ld = '1' then
                    c <= d;
                else
                    c <= c + 1;
                end if;
            end if;
        end if;
    end process;

    -- output
    count <= c;
end architecture behavioral;